package it.amattioli.encapsulate.browsers;

import it.amattioli.dominate.Described;
import it.amattioli.dominate.Entity;
import it.amattioli.dominate.util.PropertyChangeEmitter;
import it.amattioli.encapsulate.dates.ConventionalTimeInterval;
import it.amattioli.encapsulate.dates.Day;
import it.amattioli.encapsulate.dates.Hour;
import it.amattioli.encapsulate.dates.Month;
import it.amattioli.encapsulate.dates.QuarterOfDay;
import it.amattioli.encapsulate.dates.TimeInterval;
import it.amattioli.encapsulate.dates.Week;

import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Date;
import java.util.ResourceBundle;

public interface CalendarBrowser<I extends Serializable, T extends Entity<I>> extends PropertyChangeEmitter {
	
	public enum IntervalType implements Described {
		HOUR(Hour.class),
		QUARTER_OF_DAY(QuarterOfDay.class),
		DAY(Day.class),
		WEEK(Week.class),
		MONTH(Month.class);
		
		private Class<? extends ConventionalTimeInterval> intervalClass;
		
		private IntervalType(Class<? extends ConventionalTimeInterval> intervalClass) {
			this.intervalClass = intervalClass;
		}
		
		private static final ResourceBundle DESCRIPTIONS = ResourceBundle.getBundle("it.amattioli.encapsulate.browsers.IntervalTypeDescriptions");

		@Override
		public String getDescription() {
			return DESCRIPTIONS.getString(this.name());
		}

		public ConventionalTimeInterval fromDate(Date d) {
			try {
				Constructor<? extends ConventionalTimeInterval> constructor = intervalClass.getConstructor(Date.class);
				return constructor.newInstance(d);
			} catch(NoSuchMethodException e) {
				throw new RuntimeException(e);
			} catch(InvocationTargetException e) {
				throw new RuntimeException(e);
			} catch(InstantiationException e) {
				throw new RuntimeException(e);
			} catch(IllegalAccessException e) {
				throw new RuntimeException(e);
			}
		}
		
		public ConventionalTimeInterval fromInterval(TimeInterval i) {
			return fromDate(i.getLow());
		}
		
		public boolean hasPrevious() {
			return ordinal() > 0;
		}

		public IntervalType previous() {
			if (hasPrevious()) {
				return values()[ordinal() - 1];
			} else {
				return null;
			}
		}

	}
	
	public void setIntervalType(IntervalType view);
	
	public IntervalType getIntervalType();
	
	public TimeInterval getLimits();
	
	public void setLimits(TimeInterval limits);
	
	public boolean hasNextInterval();
	
	public void nextInterval();
	
	public boolean hasPreviousInterval();
	
	public void previousInterval();
	
	public void position(Date time);
	
	public void position(TimeInterval i);
	
	public void startPosition();
	
	public ConventionalTimeInterval getInterval();
	
	//public Map<Day, Collection<T>> getList();
	
	public Collection<T> getList();
	
	public CalendarBrowser<I,T> subBrowser(IntervalType type);

	public boolean hasSubBrowser();

	public CalendarBrowser<I,T> subBrowser();
	
	public void addPropertyChangeListener(PropertyChangeListener listener);
	
	public void removePropertyChangeListener(PropertyChangeListener listener);

}
