package it.amattioli.encapsulate.dates;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

public class ConventionalDuration implements Duration {
	public static enum Type {
		DAY(Calendar.DAY_OF_MONTH),
		WEEK(Calendar.WEEK_OF_YEAR),
		MONTH(Calendar.MONTH),
		YEAR(Calendar.YEAR);
		
		private int calendarField;
		
		private Type(int calendarField) {
			this.calendarField = calendarField;
		}
		
	}
	
	public static ConventionalDuration ONE_YEAR = new ConventionalDuration(Type.YEAR, 1);
	public static ConventionalDuration ONE_MONTH = new ConventionalDuration(Type.MONTH, 1);
	public static ConventionalDuration ONE_WEEK = new ConventionalDuration(Type.WEEK, 1);
	public static ConventionalDuration ONE_DAY = new ConventionalDuration(Type.DAY, 1);
	
	private Type type;
	private int amount;
	
	public ConventionalDuration(Type type, int amount) {
		this.type = type;
		this.amount = amount;
	}
	
	@Override
	public Date after(Date begin) {
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(begin);
		cal.add(type.calendarField, amount);
		return cal.getTime();
	}

	@Override
	public Date before(Date begin) {
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(begin);
		cal.add(type.calendarField, -amount);
		return cal.getTime();
	}
	
	public Duration plus(Duration d) {
		return new CompositeDuration(this, d);
	}

}
