package it.amattioli.encapsulate.dates;

import it.amattioli.encapsulate.range.Discrete;

import java.io.Serializable;
import java.util.*;
import java.text.*;

/**
 * An instance of this class represents a time interval corresponding to a day,
 * from 00:00 to 24:00.
 * As the concept of 00:00 and 24:00 is Timezone dependent, the same will be for
 * instances of this class. 
 * 
 */
public class Day extends ConventionalTimeInterval implements Discrete<Day>, Serializable, Cloneable {
	private static final long serialVersionUID = 1L;

	// private Date start;
	// private Calendar cal;
	/*
	 * private Calendar getCalendar() { if (cal == null) { cal =
	 * Calendar.getInstance(); cal.setTime(start); } return cal; }
	 * 
	 * private void setCalendar(Calendar cal) { this.cal = cal; start =
	 * cal.getTime(); }
	 */
	private Day() {
		this(new Date());
	}

	private Day(Calendar cal) {
		setCalendar(cal);
	}

	/**
	 * Construct the day that contains the passed time point in the current
	 * time zone. 
	 * 
	 * @param date
	 */
	public Day(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		setCalendar(cal);
	}

	/**
	 * Construct a day given day, month and year in the current time zone
	 * 
	 * @param day
	 *            
	 * @param month
	 *            
	 * @param year
	 *            
	 */
	public Day(int day, int month, int year) {
		Calendar cal = Calendar.getInstance();
		cal.clear();
		cal.set(Calendar.DAY_OF_MONTH, day);
		cal.set(Calendar.MONTH, month);
		cal.set(Calendar.YEAR, year);
		setCalendar(cal);
	}

	/**
	 * Costruisce un Day che rappresenta la giornata odierna.
	 */
	public static Day today() {
		return new Day();
	}

	public static Day yesterday() {
		return (Day) today().previous();
	}

	public static Day tomorrow() {
		return (Day) today().next();
	}

	/**
	 * The day of the month 
	 * 
	 */
	public int getDayOfMonth() {
		return getCalendar().get(Calendar.DAY_OF_MONTH);
	}

	/**
	 * The month
	 */
	public int getMonth() {
		return getCalendar().get(Calendar.MONTH);
	}

	/**
	 * The year
	 */
	public int getYear() {
		return getCalendar().get(Calendar.YEAR);
	}

	/**
	 * Returns the day immediately preceding this.
	 */
	public Day previous() {
		Calendar precCal = (Calendar) getCalendar().clone();
		precCal.add(Calendar.DATE, -1);
		return new Day(precCal);
	}

	/**
	 * Returns the day immediately following this.
	 */
	public Day next() {
		Calendar succCal = (Calendar) getCalendar().clone();
		succCal.add(Calendar.DATE, 1);
		return new Day(succCal);
	}

	/**
	 * Returns the time point that is the end of this day
	 * 
	 */
	public Date getEndTime() {
		Calendar succCal = (Calendar) getCalendar().clone();
		succCal.add(Calendar.DATE, 1);
		return succCal.getTime();
	}

	@Override
	public int compareTo(Day d) {
		return this.getInitTime().compareTo(d.getInitTime());
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof Day) {
			Day d = (Day) o;
			return this.getInitTime().equals(d.getInitTime());
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return getInitTime().hashCode();
	}

	@Override
	public Day clone() {
		return new Day((Calendar) getCalendar().clone());
	}

	@Override
	public String toString() {
		DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
		return format.format(getInitTime());
	}
	
	@Override
	public Duration getDuration() {
		return ConventionalDuration.ONE_DAY;
	}
}
