package it.amattioli.encapsulate.dates.hibernate;

import it.amattioli.dominate.hibernate.filters.HibernateAttributeFilter;
import it.amattioli.encapsulate.dates.TimeInterval;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Restrictions;

public class DateFilter implements HibernateAttributeFilter<TimeInterval> {
    private static final String LOW = "Low";
    private static final String HIGH = "High";
    private TimeInterval value;
    private String alias;
    private String attributeName;

    public DateFilter(String attributeName) {
        this.attributeName = attributeName;
    }
    
    public DateFilter(String alias, String attributeName) {
    	this.alias = alias;
        this.attributeName = attributeName;
    }

    public TimeInterval get() {
        return value;
    }

    public void set(TimeInterval value) {
        this.value = value;
    }

    protected boolean wasSet() {
        return get() != null;
    }

    public void addCriteria(Criteria crit) {
        if (wasSet()) {
        	crit.add(Restrictions.ge(attributeName, get().getLow()));
        	crit.add(Restrictions.lt(attributeName, get().getHigh()));
        }
    }

    public void addHqlCondition(StringBuffer buffer) {
        if (wasSet()) {
            if (buffer.length() == 0) {
                buffer.append("where ");
            } else {
                buffer.append("and ");
            }
            buffer.append("(");
            if (alias != null) {
            	buffer.append(alias).append('.');
            }
            buffer.append(attributeName)
                  .append(" >= :")
                  .append(attributeName).append(LOW)
                  .append(" and ")
                  .append(attributeName)
                  .append(" < :")
                  .append(attributeName).append(HIGH)
                  .append(")");
        }
    }

    public void setHqlParam(Query query) {
        if (wasSet()) {
            query.setParameter(attributeName+LOW, get().getLow());
            query.setParameter(attributeName+HIGH, get().getHigh());
        }
    }

}
