package it.amattioli.encapsulate.range;

public abstract class AbstractRange<T extends Comparable<? super T>> implements Range<T> {

	public boolean includes(T test) {
		return (!isLowBounded() || getLow().compareTo(test) <= 0) 
		    && (!isHighBounded() || getHigh().compareTo(test) >= 0);
	}

	public boolean contains(Range<? extends T> r) {
		return (!this.isLowBounded() || (r.isLowBounded() && r.getLow().compareTo(getLow()) >= 0))
			&& (!this.isHighBounded() || (r.isHighBounded() && r.getHigh().compareTo(getHigh()) <= 0));
	}

	public boolean overlaps(Range<? extends T> r) {
		return (!r.isLowBounded() || !this.isHighBounded() || (this.isHighBounded() && this.getHigh().compareTo(r.getLow()) >= 0))
			&& (!this.isLowBounded() || !r.isHighBounded() || (r.isHighBounded() && r.getHigh().compareTo(this.getLow()) >= 0));
	}

	public boolean hasSameLow(Range<? extends T> r) {
		return (!this.isLowBounded() && !r.isLowBounded()) || getLow().equals(r.getLow());
	}

	public boolean hasSameHigh(Range<? extends T> r) {
		return (!this.isHighBounded() && !r.isHighBounded()) || getHigh().equals(r.getHigh());
	}

	@SuppressWarnings("unchecked")
	public boolean equals(Object obj) {
		boolean result = false;
		if (obj instanceof Range) {
			Range<T> r = (Range<T>) obj;
			result = hasSameLow(r) && hasSameHigh(r);
		}
		return result;
	}

	public int hashCode() {
		int result = 17;
		result = 37 * result;
		if (isLowBounded()) {
			result += getLow().hashCode();
		}
		result = 37 * result;
		if (isHighBounded()) {
			result += getHigh().hashCode();
		}
		return result;
	}

	public String toString() {
		return "(" + getLow().toString() + "," + getHigh().toString() + ")";
	}

	/**
	 * Crea un nuovo range compatibile con questo.
	 * 
	 * @param <U>
	 * @param low
	 * @param high
	 * @return
	 */
	protected abstract <U extends Range<? extends T>> U newRange(T low, T high);

}
