package it.amattioli.encapsulate.range.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.specifications.ChainedSpecification;
import it.amattioli.dominate.specifications.SpecificationsConfig;
import it.amattioli.encapsulate.range.Range;

public abstract class ComparableSpecification<T extends Entity<?>, N extends Comparable<? super N>> extends ChainedSpecification<T> {
	private String propertyName;
	private Range<N> value;
	
	public static <T extends Entity<?>, N extends Comparable<? super N>> ComparableSpecification<T,N> newInstance(String propertyName) {
		ComparableSpecification<T,N> result = null;
		for (Class<? extends ComparableSpecification> curr: SpecificationsConfig.getSpecificationClasses(ComparableSpecification.class)) {
			Constructor<? extends ComparableSpecification> c;
			try {
				c = curr.getConstructor(String.class, ComparableSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (ComparableSpecification<T,N>)c.newInstance(propertyName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public ComparableSpecification(String propertyName) {
		this.propertyName = propertyName;
	}
	
	public ComparableSpecification(String propertyName, ComparableSpecification<T,N> chained) {
		super(chained);
		this.propertyName = propertyName;
	}

	protected String getPropertyName() {
		return propertyName;
	}

	public void setValue(Range<N> value) {
		this.value = value;
		if (getNextInChain() != null) {
			((ComparableSpecification<T,N>)getNextInChain()).setValue(value);
		}
	}

	public Range<N> getValue() {
		return value;
	}
	
	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			N propertyValue = (N)PropertyUtils.getProperty(entity, getPropertyName());
			return getValue().includes(propertyValue);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getValue() != null;
	}
}
